/*
 * YouGrabber
 *
 * A lightweight, ncurses-based, multi-threaded command line YouTube.com
 * video downloader with proxy support, made from scratch in ANSI C.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "common.h"

int main(int argc, char **argv)
{
  GList *f_list = NULL;
  C *c_ptr = NULL;
  I *i_ptr = NULL;
  int i;

  /* set configuration */
  if (!(c_ptr = config_new()))
  {
    fprintf(stderr, "%s v%s - Copyright (C) %s <%s>\n\n"\
      "insufficient memory to store configuration settings\n\n",
      YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL);

    return EXIT_FAILURE;
  }

  if (argc < 2)
  {
    config_free(c_ptr);

    fprintf(stderr, "%s v%s - Copyright (C) %s <%s>\n\n"\
      "there are no downloads to process\n\n",
      YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL);

    return EXIT_SUCCESS;
  }

  /* login into youtube and get cookie */
  config_set_youtube_login_status(c_ptr, download_cookie(c_ptr));

  /* set downloads in progress */
  dip_set(argc-1);

  interface_nc_start();

  /* create a frame for each URL */
  for (i = 1; i < argc; ++i)
  {
    F *f_ptr = NULL;

    f_ptr = interface_frame_new(download_new(c_ptr, argv[i], download_grab));

    if (f_ptr != NULL)
      f_list = g_list_append(f_list, f_ptr);
    else
      dip_rem();
  }

  /* create user interface */
  if (!(i_ptr = interface_new(c_ptr, f_list)))
  {
    interface_nc_stop();

    g_list_foreach(f_list, interface_frame_free, NULL);
    g_list_free(f_list);

    config_free(c_ptr);

    fprintf(stderr, "%s v%s - Copyright (C) %s <%s>\n\n"\
      "insufficient memory to start user interface\n\n",
      YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL);

    return EXIT_FAILURE;
  }

  interface_main_loop(i_ptr);

  interface_nc_stop();

  interface_free(i_ptr);

  return EXIT_SUCCESS;
}
